<?php

namespace Quivers\Integration\Model\Config\Source;
use Quivers\Integration\LogTracking;
class MarketPlaceConfig implements \Magento\Framework\Option\ArrayInterface
{
	protected $curlClient;
	protected $scopeConfig;
	protected $baseUrl;

	protected $baseTestUrl = "https://cloudhub.quiversdemo.com/";
	protected $baseProdUrl = "https://cloudhub-internal.quivers.com/";
	protected $marketPlaceApi = "/api/v1/private/ProductGroups/MyProductGroups?refId=";

    const XML_QUIVERS_API_KEY = 'quivers/parameters/quivers_api_key';
    const XML_QUIVERS_BUSINESS_REFID = 'quivers/parameters/merchant_ref_id';
    const XML_QUIVERS_SANDBOX = 'quivers/parameters/quivers_sanndbox';

	public function __construct(
		\Magento\Framework\HTTP\Client\Curl $curl,
		\Magento\Framework\App\Request\Http $request,
        \Magento\Framework\App\Config\ScopeConfigInterface $scopeConfig
	) {
		$this->curlClient = $curl;
		$this->request = $request;
		$this->scopeConfig = $scopeConfig;
	}

	public function getMarketPlaces() {
        $websiteScope = \Magento\Store\Model\ScopeInterface::SCOPE_WEBSITE;
        $storeScope = \Magento\Store\Model\ScopeInterface::SCOPE_STORE;
		$debug_mod = $this->scopeConfig->getValue('quivers/parameters/quivers_debug_mode', $storeScope);
        $log = new LogTracking($this->scopeConfig);
        $websiteId = (int) $this->request->getParam('website',0);
        $storeId = (int) $this->request->getParam('store',0);
		if($websiteId){
			$quiversAPIKey = $this->scopeConfig->getValue(self::XML_QUIVERS_API_KEY, $websiteScope, $websiteId);
            $quiversSandbox = $this->scopeConfig->getValue(self::XML_QUIVERS_SANDBOX, $websiteScope, $websiteId);
        	$quiversBusinessRefId = $this->scopeConfig->getValue(self::XML_QUIVERS_BUSINESS_REFID, $websiteScope, $websiteId);
		}else{
			$quiversAPIKey = $this->scopeConfig->getValue(self::XML_QUIVERS_API_KEY, $storeScope, $storeId);
            $quiversSandbox = $this->scopeConfig->getValue(self::XML_QUIVERS_SANDBOX, $storeScope, $storeId);
        	$quiversBusinessRefId = $this->scopeConfig->getValue(self::XML_QUIVERS_BUSINESS_REFID, $storeScope, $storeId);
		}
		if($debug_mod){
            $log->debug_mode_on(
				$debug_mod,'WebsiteId - '.$websiteId,
				null,null,null); 
        }
		// get market places by calling the market place api
		if($quiversSandbox==1) $this->baseUrl = $this->baseTestUrl;
		else $this->baseUrl = $this->baseProdUrl;

		$profileUrl = $this->baseUrl.$this->marketPlaceApi.$quiversBusinessRefId;
        $this->curlClient->addHeader("Content-Type","application/json");
        $this->curlClient->addHeader("Authorization","apikey ".$quiversAPIKey);
        $this->curlClient->get($profileUrl);

		$response = json_decode($this->curlClient->getBody());
		$marketPlaces = [];
		array_push($marketPlaces, (object)['id' => null, 'name' => "Select Storefront" ]);
		if($this->curlClient->getStatus()==200 && is_array($response)) {
			if(count($response)>0) {
				foreach($response as $res) {
					if($res->Type=="Marketplace") {
						array_push($marketPlaces, (object)['id' => $res->Id, 'name' => $res->Name ]);
					}
				}
			}
		}
		return $marketPlaces;
	}

	public function toOptionArray() {

		$allMarketPlaces = $this->getMarketPlaces();

		$options = [];
		if(count($allMarketPlaces)>0) {
			foreach($allMarketPlaces as $mPlace) {
				$options[] = ['value' => $mPlace->id, 'label' => __($mPlace->name)];
			}
		}
		return $options;
	}
}