<?php
namespace Quivers\Integration\Model;

use Stripe\Stripe;
use \Magento\Customer\Model\Session as customerSession;
use Quivers\Integration\LogTracking;

class QuiversStripe extends \Magento\Payment\Model\Method\Cc
{

    protected $_code = 'quivers_integration';
    protected $_canAuthorize = true;
    protected $_isGateway = true;
    protected $curlClient;
    protected $_stripeApi = false;
    protected $_scopeConfig;
    protected $objectManager;
    protected $orderRepository;
 
    public function __construct(
     
     
        \Magento\Framework\Model\Context $context,
        \Magento\Framework\Registry $registry,
        \Magento\Framework\Api\ExtensionAttributesFactory $extensionFactory,
        \Magento\Framework\Api\AttributeValueFactory $customAttributeFactory,
        \Magento\Payment\Helper\Data $paymentData,
        \Magento\Framework\App\Config\ScopeConfigInterface $scopeConfig,
        \Magento\Payment\Model\Method\Logger $logger,
        \Magento\Framework\Module\ModuleListInterface $moduleList,
        \Magento\Framework\Stdlib\DateTime\TimezoneInterface $localeDate,
        \Magento\Framework\Message\ManagerInterface $messageManager,
        customerSession $customerSession,
        \Magento\Checkout\Model\Session $checkoutSession
       
    ) {
        parent::__construct(
            $context,
            $registry,
            $extensionFactory,
            $customAttributeFactory,
            $paymentData,
            $scopeConfig,
            $logger,
            $moduleList,
            $localeDate,
        );
        $this->_scopeConfig = $scopeConfig;
        $this->_checkoutSession = $checkoutSession;
        $storeScope = \Magento\Store\Model\ScopeInterface::SCOPE_STORE;
        $this->curlClient = \Magento\Framework\App\ObjectManager::getInstance()->get(\Magento\Framework\HTTP\Client\Curl::class);
        $this->_logger = $logger;


    //   $this->orderModel = $orderModel;

        // get the configured values in payment configuration form.
        $secret_key = $this->_scopeConfig->getValue("payment/quivers_integration/secret_key", $storeScope);
        $this->_minAmount = $this->_scopeConfig->getValue("payment/quivers_integration/min_order_total", $storeScope);
        $this->customerSession = $customerSession;
        $this->messageManager = $messageManager;
        Stripe::setApiKey($secret_key);
    }

    public function assignData(\Magento\Framework\DataObject $data)
    {   
        parent::assignData($data);
        $storeScope = \Magento\Store\Model\ScopeInterface::SCOPE_STORE;
        $debug_mod =$this->_scopeConfig->getValue('quivers/parameters/quivers_debug_mode', $storeScope);
        $log = new LogTracking($this->_scopeConfig);
        $objectManager = \Magento\Framework\App\ObjectManager::getInstance();
        $cart = $objectManager->get('\Magento\Checkout\Model\Cart');
        $quote = $cart->getQuote();
        $shipping_address = $quote->getShippingAddress();
        $phone = $shipping_address->getTelephone();

        $additionalData = $data->getAdditionalData();
        $info = $this->getInfoInstance();
      
        $log->debug_mode_on($debug_mod,
        'assign data start- '.$phone,
        null, null ,null);

        if (is_array($additionalData))
            $data->setData(array_merge($data->getData(), $additionalData));



        $token = $data['cc_stripejs_token'];
        $saveMyCard = $data['cc_save_card'];
        $captchatoken = $data['captcha_token'];
        $payment_method_id = $data['payment_method'];
        // assigning data received from payment form so that it can be used
        // later in authorize method.
        $info->setAdditionalInformation('cc_stripejs_token', $token);
        $info->setAdditionalInformation('captcha_token', $captchatoken);
        $info->setAdditionalInformation('cc_save_card', $saveMyCard);
        $info->setAdditionalInformation('payment_method', $payment_method_id);
        // logs


        $logData = [
            "additionalData"=> $additionalData,
            "data" => $data->getData(),
            "token"=> $token,
            "saveMyCard" => $saveMyCard,
            "captchatoken" => $captchatoken
        ];
        $log->debug_mode_on($debug_mod,
                    'assign data quiversstripe.php- '.$phone,
                    null, null ,$logData);
        return $this;
    }


    public function authorize(\Magento\Payment\Model\InfoInterface $payment, $amount)
    {
        // get saved data
        $objectManager = \Magento\Framework\App\ObjectManager::getInstance();
        $scopeConfig = \Magento\Framework\App\ObjectManager::getInstance()->get(\Magento\Framework\App\Config\ScopeConfigInterface::class);
        $startTime = (int) microtime(true);
        $log = new LogTracking($this->_scopeConfig);
        $storeScope = \Magento\Store\Model\ScopeInterface::SCOPE_STORE;
        $debug_mod =$this->_scopeConfig->getValue('quivers/parameters/quivers_debug_mode', $storeScope);
        $payment_token = $payment->getAdditionalInformation('cc_stripejs_token');
        $saveMyCard = $payment->getAdditionalInformation('cc_save_card');
        $token = $payment->getAdditionalInformation('captcha_token');
        $payment_Method_Id = $payment->getAdditionalInformation('payment_method');
        $order = $payment->getOrder();
        $billing = $order->getBillingAddress();
        $customerEmail = $billing->getEmail();
        $log->debug_mode_on($debug_mod,
        'authorize customer quiversstripe.php- '.$customerEmail,
        null, null ,null);     

        $objectManager = \Magento\Framework\App\ObjectManager::getInstance();
        $cart = $objectManager->get('\Magento\Checkout\Model\Cart');
        $quote = $cart->getQuote();
        if(!$customerEmail){
            throw new \Magento\Framework\Validator\Exception(__(' No Email id provided'));
        }

        try {
            //check if user with same email already exists in Stripe account.
            $customer = false;
            $customersList = \Stripe\Customer::all(array(
                "limit" => 1,
                "email" => $customerEmail,
            ));

            $log->debug_mode_on($debug_mod,
            'authorize customer inside try  quiversstripe.php- '.$customerEmail,
            null, null ,count($customersList["data"]));
            // create a new user if it doesn't exists.
            if(count($customersList["data"])>0) {
                $customer = $customersList["data"][0];
            }
            //debug mode
            if($debug_mod){
                $log->debug_mode_on($debug_mod,
                    'authorize customer check quiversstripe.php- '.$customerEmail,
                    null, null ,$customer);
            }
            if(!$customer){
                    $customer = \Stripe\Customer::create(array(
                    'source' => $payment_token,
                    'email' => $customerEmail,
                    ));
                        // //debug mode
                if($debug_mod){
                    $log->debug_mode_on($debug_mod,
                        'authorize inside customer create quiversstripe.php- '.$customerEmail,
                        null, null ,$customer);
                }            
            }
            if ($saveMyCard)
            {  // checked
               try {
                   $customer->sources->create(array("source" => $payment_token));
                   }
               catch(\Exception $e){
                $log->debug_mode_on($debug_mod,
                'Catch  "QuiversStripe.php -" '.$quote->getBillingAddress()->getEmail(),
                null,  null,$e->getMessage());
                    }
            }
            if(!$customer){
                throw new \Magento\Framework\Validator\Exception(__(' Unable to Fetch Stripe Customer Details'));
            }
            $customerId = isset($customer["id"])?$customer["id"]:null;
            $customer = $this->customerSession->getCustomer();
        
            if($debug_mod){
                $log->debug_mode_on($debug_mod,
                    'authorize inside customer data from session  quiversstripe.php- '.$customerEmail,
                    null, null ,$customer);
            }
            if(!$customer){
                throw new \Magento\Framework\Validator\Exception(__(' Unable to Fetch Customer Details'));
            }
            $myCustomerId = $customer->getId();

            if($debug_mod){
                $log->debug_mode_on($debug_mod,
                    'authorize quiversstripe.php- '.$customerEmail,
                    null, null ,$myCustomerId);
            }

            if(isset($customerId) && $myCustomerId) {
                if(!$customer->getQuiversStripeCustomerId()){
                    $customer->setAttributeSetId(\Magento\Customer\Api\CustomerMetadataInterface::ATTRIBUTE_SET_ID_CUSTOMER);
                    $customer->setQuiversStripeCustomerId($customerId);  
                }
             
                if ($saveMyCard) {  // checked
                    // add current card.             
                    $savedCardIds = explode(',', $customer->getQuiversStripeCardIds() ?? '');
                    if(count($savedCardIds)==10){
                        unset($savedCardIds[1]);
                    }
                    $savedCardIds[] = $payment_token;  
                    $customer->setQuiversStripeCardIds(implode(",", $savedCardIds));
                }
                $customer->save();
            }
        } catch (\Exception $e) {
            $log->debug_mode_on($debug_mod,
			'Catch  "QuiversStripe.php -"Please verify the details before trying again  '.$quote->getBillingAddress()->getEmail(),
			null,  null,$e->getMessage());
            throw new \Magento\Framework\Validator\Exception(__($e->getMessage().' Please verify the details before trying again. If the issue persists please contact Quivers at 805-250-4555'));
        }
        $objectManager = \Magento\Framework\App\ObjectManager::getInstance();
        $customer_session=$objectManager->get('\Magento\Customer\Model\Session');
        $secret = isset($customer_session->getSiteKey()['site_key_secret'])?$customer_session->getSiteKey()['site_key_secret']:'NO-SECRET_KEY';
        if($debug_mod){

            $log->debug_mode_on($debug_mod,

                'site_key_secret inside quiversstripe.php- '.$customerEmail,

                null, null ,$secret);

        }
        if(isset($token) && $token!=null && isset($secret)):
            //get verify response data
            function file_get_contents_curl( $url ) {

                $ch = curl_init();
              
                curl_setopt( $ch, CURLOPT_AUTOREFERER, TRUE );
                curl_setopt( $ch, CURLOPT_HEADER, 0 );
                curl_setopt( $ch, CURLOPT_RETURNTRANSFER, 1 );
                curl_setopt( $ch, CURLOPT_URL, $url );
                curl_setopt( $ch, CURLOPT_FOLLOWLOCATION, TRUE );
              
                $data = curl_exec( $ch );
                curl_close( $ch );
              
                return $data;
              
            }

            $verifyResponse = file_get_contents_curl('https://www.google.com/recaptcha/api/siteverify?secret='.$secret.'&response='.$token.'&remoteip='.$_SERVER["REMOTE_ADDR"]);
            $responseData = json_decode($verifyResponse);
            $log->captcha_data($customerEmail,$token,$responseData->success);
                if($responseData->success):
                    // $charge = $this->createCharge($customerId, $payment, $amount);
                else:
                    $responseData = json_decode(json_encode($responseData),true);
                    if(isset($responseData["error-codes"][0]) && $responseData["error-codes"][0]=='invalid-input-secret') :
                         //$charge = $this->createCharge($customerId, $payment, $amount);
                    else:
                        $message = "Please reload your page  and try again. If the issue still persists,please contact 'enterprise@quivers.com' for further assistance";
                        throw new \Magento\Framework\Validator\Exception(__($message));
                    endif;
                endif;
        endif;
        try
        {           
            $payment_intent_id=$this->customerSession->getPaymentIntentId();
            #retieve payment intent to get charge id and status
            $payment_intent=\Stripe\PaymentIntent::retrieve($payment_intent_id);

            $objectManager = \Magento\Framework\App\ObjectManager::getInstance();
			$storeManager = $objectManager->get('\Magento\Store\Model\StoreManagerInterface');
			$baseUrl=$storeManager->getStore()->getBaseUrl();

            $this->customerSession->setMyValue($payment_intent_id);
          
            $secret_key = $scopeConfig->getValue("payment/quivers_integration/secret_key", $storeScope);

            $stripe= new \Stripe\StripeClient($secret_key);

            $checkout_url=$baseUrl."checkout/onepage/success";
            $payment_intent= $stripe->paymentIntents->confirm(
                $payment_intent_id,
                [
                    'return_url' =>   $checkout_url, 
                    'source' =>   $payment_token
                    //'payment_method' => $payment_Method_Id
                ]
            ); 
            if ($payment_intent["status"]== "requires_action"|| $payment_intent["status"]== "requires_source_action"){
                return;   
            } 
            else if ($payment_intent["status"]== "requires_capture"){
                $payment_intent = $stripe->paymentIntents->capture(
                    $payment_intent_id,
                    []
                );
            }
            #set get charge id and status to payemnt object

            $order_2 = $payment->getOrder();
            $payment
                ->setTransactionId($payment_intent->charges->data[0]->id)
                ->setIsTransactionClosed(0);
            #sending logs
            $log->debug_mode_on($debug_mod,
        'Finding charge id '.$order->getCustomerEmail(),
        null, null ,$payment_intent->charges->data[0]->id);    
            $order_data = [ "Customer Id"=>$customerId,
                            "Email"=>$order->getCustomerEmail(),
                           // "Finalamount"=>$payment->getAdditionalInformation('QuiversStripeChargeAmount'),
                            "Finalamount"=>$payment->getAdditionalInformation('QuiversStripeChargeAmount'),
                            "Magento2Amount" =>$amount        
                        ];
                        
            $response = ["Response"=>$payment_intent->charges->data[0], "Amount"=>$amount,'source_id'=>$payment_token];
            $log->final_payment_intent("stripe", date('r', $startTime), $payment_intent);
            $log->payment_request($order_data, "stripe", date('r', $startTime), $response, $payment_intent->status);
            $this->customerSession->unsetPaymentIntentId();
            $this->customerSession->unsetTotalAmount();
            return $this;
        }
        catch (\Exception $e) {
            $log->debug_mode_on($debug_mod,
			'Catch  "QuiversStripe.php -"Please verify the details before trying again  '.$quote->getBillingAddress()->getEmail(),
			null,  null,$e->getMessage());
            $objectManager = \Magento\Framework\App\ObjectManager::getInstance(); 
            $customer_session=$objectManager->get('\Magento\Customer\Model\Session');
            $customer_session->setPaymentError($e->getMessage());
            return $this;
        }
    }
    
    public function formatCurrencyAmount($amount, $currency)
    {
        // Stripe takes amounts as the smallest currency unit. ...

        if (empty($amount)) {
            return null;
        }
        $multiple = 100;
        if (in_array(
            $currency,
            array('BIF', 'DJF', 'JPY', 'KRW', 'PYG', 'VND', 'XAF', 'XPF', 'CLP', 'GNF', 'KMF', 'MGA', 'RWF', 'VUV', 'XOF'),
            true
        )) {
            $multiple = 1;
        }

        return sprintf('%01.0f', (float)$amount * $multiple);
    }


    
    public function isAvailable(\Magento\Quote\Api\Data\CartInterface $quote = null)
    {   
        if ($quote && $quote->getBaseGrandTotal() <= $this->_minAmount) {
            return false;
        }
        $storeScope = \Magento\Store\Model\ScopeInterface::SCOPE_STORE;
        $secretKey = $this->_scopeConfig->getValue("payment/quivers_integration/secret_key", $storeScope);
        $activekey = $this->_scopeConfig->getValue("payment/quivers_integration/active", $storeScope);
        $active = $this->_scopeConfig->getValue("quivers/parameters/active", $storeScope);
        if (!$secretKey  or $active==0) {
            return false;
        }

        return $activekey;
    }
    public function canUseForCurrency($currencyCode)
    {   
        return true;
    }

    public function validate()
    {   
        // validation is already handled by stripejs
        return true;
    }



}