<?php

namespace Quivers\Integration\Observer;

use Magento\Framework\Event\ObserverInterface;
use Magento\Framework\Event\Observer as EventObserver;
use Psr\Log\LoggerInterface as Logger;
use Magento\Framework\Exception\CouldNotSaveException;
use Quivers\Integration\LogTracking;

class CurrencyMappingObserver implements ObserverInterface
{
    /**
     * @var Logger
     */
    protected $logger;
    public $layout;
    protected $curlClient;
    protected $scopeConfig;
    protected $_responseFactory;
    public $_backendUrl;
    protected $configWriter;

    /**
     * @param Logger $logger
     */
    
    const XML_ENABLED = 'quivers/parameters/active';
    const XML_QUIVERS_SANDBOX = 'quivers/parameters/quivers_sanndbox';
    const XML_USD_MARKETPLACE_ID = 'quivers_tax/marketplaces/usdMarketPlaceId';
    const XML_EUR_MARKETPLACE_ID = 'quivers_tax/marketplaces/eurMarketPlaceId';
    const XML_CAD_MARKETPLACE_ID = 'quivers_tax/marketplaces/cadMarketPlaceId';
    const XML_AUD_MARKETPLACE_ID ='quivers_tax/marketplaces/audMarketPlaceId';
    const XML_JPY_MARKETPLACE_ID = 'quivers_tax/marketplaces/jpyMarketPlaceId';
    const XML_GBP_MARKETPLACE_ID = 'quivers_tax/marketplaces/gbpMarketPlaceId';
    const XML_QUIVERS_UUID = 'quivers/parameters/quivers_uuid';
    const XML_USD_CLAIMINGGROUPS = 'quivers_tax/marketplaces/usdClaimingGroups';
    const XML_EUR_CLAIMINGGROUPS = 'quivers_tax/marketplaces/eurClaimingGroups';
    const XML_CAD_CLAIMINGGROUPS = 'quivers_tax/marketplaces/cadClaimingGroups';
    const XML_AUD_CLAIMINGGROUPS = 'quivers_tax/marketplaces/audClaimingGroups';
    const XML_JPY_CLAIMINGGROUPS = 'quivers_tax/marketplaces/jpyClaimingGroups';
    const XML_GBP_CLAIMINGGROUPS = 'quivers_tax/marketplaces/gbpClaimingGroups';
    const XML_QUIVERS_STATUS_LABEL = 'quivers/parameters/quivers_status_label';


    protected $baseUrl = 'https://middleware.quiversdemo.com/profile';

    public function __construct(

        Logger $logger,
        \Magento\Framework\HTTP\Client\Curl $curl,
        \Magento\Framework\App\Config\ScopeConfigInterface $scopeConfig,
        \Magento\Framework\App\ResponseFactory $responseFactory,
        \Magento\Framework\View\LayoutInterface $layout,
        \Magento\Backend\Model\UrlInterface $backendUrl,
        \Magento\Framework\App\Request\Http $request,
        \Magento\Framework\App\Config\Storage\WriterInterface $configWriter,
        \Magento\Store\Model\StoreManagerInterface $storeManager,
        \Magento\Framework\Message\ManagerInterface $messageManager
    ){
        $this->curlClient = $curl;
        $this->scopeConfig = $scopeConfig;
        $this->_responseFactory = $responseFactory;
        $this->layout = $layout;
        $this->_backendUrl = $backendUrl;
        $this->request = $request;
        $this->configWriter = $configWriter;
        $this->storeManager = $storeManager;
        $this->messageManager = $messageManager;


    }

    public function execute (EventObserver $observer) {
        $log = new LogTracking($this->scopeConfig);

        $storeScope = \Magento\Store\Model\ScopeInterface::SCOPE_STORE;
        $websiteScope = \Magento\Store\Model\ScopeInterface::SCOPE_WEBSITE;
        $scopeConfig = \Magento\Framework\App\ObjectManager::getInstance()->get(\Magento\Framework\App\Config\ScopeConfigInterface::class);
        $debug_mod = $scopeConfig->getValue('quivers/parameters/quivers_debug_mode', $storeScope);
        $log = new LogTracking($scopeConfig);

        $usdMarketPlaceId = $this->scopeConfig->getValue(self::XML_USD_MARKETPLACE_ID, $storeScope);
        $eurMarketPlaceId = $this->scopeConfig->getValue(self::XML_EUR_MARKETPLACE_ID, $storeScope);
        $cadMarketPlaceId = $this->scopeConfig->getValue(self::XML_CAD_MARKETPLACE_ID, $storeScope);
        $audMarketPlaceId = $this->scopeConfig->getValue(self::XML_AUD_MARKETPLACE_ID, $storeScope);
        $jpyMarketPlaceId = $this->scopeConfig->getValue(self::XML_JPY_MARKETPLACE_ID, $storeScope);
        $gbpMarketPlaceId = $this->scopeConfig->getValue(self::XML_GBP_MARKETPLACE_ID, $storeScope);
        
        $usdClaimingGroups=$this->scopeConfig->getValue(self::XML_USD_CLAIMINGGROUPS, $storeScope);
        $eurClaimingGroups=$this->scopeConfig->getValue(self::XML_EUR_CLAIMINGGROUPS, $storeScope);
        $cadClaimingGroups=$this->scopeConfig->getValue(self::XML_CAD_CLAIMINGGROUPS, $storeScope);
        $audClaimingGroups=$this->scopeConfig->getValue(self::XML_AUD_CLAIMINGGROUPS, $storeScope);
        $jpyClaimingGroups=$this->scopeConfig->getValue(self::XML_JPY_CLAIMINGGROUPS, $storeScope);
        $gbpClaimingGroups=$this->scopeConfig->getValue(self::XML_GBP_CLAIMINGGROUPS, $storeScope);
        
        $storeId = (int) $this->request->getParam('store',0);
        $websiteId = (int) $this->request->getParam('website',0);
        if($websiteId){
            $active = $this->scopeConfig->getValue(self::XML_ENABLED, $websiteScope, $websiteId);
            $quiversSandbox = $this->scopeConfig->getValue(self::XML_QUIVERS_SANDBOX, $websiteScope, $websiteId);
            $debug_mod = $this->scopeConfig->getValue('quivers/parameters/quivers_debug_mode', $websiteScope, $websiteId);
            if($debug_mod){
                $log->debug_mode_on($debug_mod, "CurrencyMappingObserver.php execute function if condition is executing with websiteId-".$websiteId, null,null, 
                ["websiteId"=>$websiteId,
                "storeId"=> $storeId,
                "quiversSandbox" => $quiversSandbox,
                "active" => $active,
                ]
            );
        }

        }else{
            $active = $this->scopeConfig->getValue(self::XML_ENABLED, $storeScope, $storeId);
            $quiversSandbox = $this->scopeConfig->getValue(self::XML_QUIVERS_SANDBOX, $storeScope, $storeId);
            $debug_mod = $this->scopeConfig->getValue('quivers/parameters/quivers_debug_mode', $storeScope, $storeId);
            if($debug_mod){
                $log->debug_mode_on($debug_mod, "CurrencyMappingObserver.php execute function if condition is executing with storeId-".$storeId, null,null, 
                ["websiteId"=>$websiteId,
                "storeId"=> $storeId,
                "quiversSandbox" => $quiversSandbox,
                "active" => $active,
                ]
            );
        }
        }

        if($quiversSandbox == 0) {
            $this->baseUrl = "https://middleware.quivers.com/profile";
        }
        $profileUpdateUrl = $this->baseUrl . "/update/marketplaces";

        $log->debug_mode_on($debug_mod, "SaveQuiversTaxSettings", null,null, 
            ["websiteId"=>$websiteId,
            "storeId"=> $storeId,
            "quiversSandbox" => $quiversSandbox,
            "active" => $active,
            "OldSandboxValue" => $this->scopeConfig->getValue(self::XML_QUIVERS_SANDBOX, $storeScope)
            ]
        );  

        $blockObj = $this->layout->createBlock('Quivers\Integration\Block\Adminhtml\Form\Button');
        $allMarketPlaces = [];
        $claimingGroupIds=[];
        $claimingGroupIds=[
            "USD" => explode(",", $usdClaimingGroups ?? ''),
            "EUR" => explode(",", $eurClaimingGroups ?? ''),
            "CAD" => explode(",", $cadClaimingGroups ?? ''),
            "AUD" => explode(",", $audClaimingGroups ?? ''),
            "JPY" => explode(",", $jpyClaimingGroups ?? ''),
            "GBP" => explode(",", $gbpClaimingGroups ?? '')
        ];
        $marketPlaceIds = [
            "USD" => $usdMarketPlaceId,
            "CAD" => $cadMarketPlaceId,
            "EUR" => $eurMarketPlaceId,
            "AUD" => $audMarketPlaceId,
            "JPY" => $jpyMarketPlaceId,
            "GBP" => $gbpMarketPlaceId
        ];
        foreach($marketPlaceIds as $mcurrency=>$mPlaceId) {
            if($mPlaceId) {
                foreach($claimingGroupIds as $ccurrency=>$cGroupIds) {
                    if ($mcurrency==$ccurrency){
                        if ($cGroupIds==[""]){$cGroupIds=[];}
                        $allMarketPlaces[] = [
                        'marketplace_id' => $mPlaceId,
                        'currency' => $mcurrency,
                        'claiming_group_ids' => $cGroupIds         
                        ];
                    }
            }}
        }
        if(count($allMarketPlaces)>0) {
            $quiversUID = $this->scopeConfig->getValue("quivers/parameters/quivers_uuid");
            $payload = [ "marketplaces" => $allMarketPlaces , "enable_plugin" => "true" ];
            $this->curlClient->addHeader("Content-Type", "application/json");
            $this->curlClient->addHeader("uuid", $quiversUID);
            $this->curlClient->post($profileUpdateUrl, json_encode($payload));
            $response = $this->curlClient->getBody();
            if($debug_mod){
                $logData = [
                    "Url" => $profileUpdateUrl,
                    "Status" =>$this->curlClient->getStatus()
                ];
                $log->debug_mode_on($debug_mod,
                    'Marketplace On Quivers ',
                    $payload,  $response,$logData);
            }
            if($this->curlClient->getStatus()==200){
                $websiteId = (int) $this->request->getParam('website',0);
                $currentStoreId = $this->request->getParam('store', 0);
                if($debug_mod){
                    $log->debug_mode_on($debug_mod,
                    'Curl client responded with status OK',
                    null,  $response,["Get Status"=>$this->curlClient->getStatus(),"websiteid"=>$websiteId,"currentStoreId"=>$currentStoreId]);    
                    }  
                if($websiteId){
                    $url = $this->_backendUrl->getUrl("adminhtml/system_config/edit/section/quivers/website/".$websiteId);
                }else{
                    $url = $this->_backendUrl->getUrl("adminhtml/system_config/edit/section/quivers/store/".$currentStoreId);
                }
                $objectManager = \Magento\Framework\App\ObjectManager::getInstance();
                $customer_session=$objectManager->get('\Magento\Customer\Model\Session');
                if ($customer_session->getSettingStatus()!="Active"){
                $this->messageManager->addErrorMessage("Failed to connect to Quivers. Please check if the settings in the Quivers and Quivers Tax tabs are saved correctly. If the issue still persists, please contact enterprise@quivers.com for further assistance.");}
                $this->_responseFactory->create()->setRedirect($url)->sendResponse();
                return;
            }
            if($this->curlClient->getStatus() >= 400) {
                $errorMessage = 'An error occurred while updating the profile. ERROR: '.$response;
                $log->debug_mode_on($debug_mod,
                    'Curl client responded with status >=400',
                    null,  $response,["Get Status"=>$this->curlClient->getStatus(),"errorMessage"=>$errorMessage]);
                $blockObj->getBackendSession()->unsMyValue();
                   $this->configWriter->save(self::XML_QUIVERS_STATUS_LABEL, "InActive");
                throw new CouldNotSaveException(
                    __($errorMessage));
            }
        }
    }

} 
