<?php

declare(strict_types=1);

namespace Quivers\Integration\Setup\Patch\Data;

use Magento\Framework\Setup\ModuleDataSetupInterface;
use Magento\Framework\Setup\Patch\DataPatchInterface;

use Magento\Eav\Model\Entity\Attribute\ScopedAttributeInterface;
use Magento\Eav\Setup\EavSetup;
use Magento\Eav\Setup\EavSetupFactory;

use Magento\Customer\Model\Customer;
use Magento\Customer\Setup\CustomerSetupFactory;
use Magento\Framework\App\ResourceConnection;
use Magento\Framework\Setup\Patch\PatchVersionInterface;

/**
* Patch is mechanism, that allows to do atomic upgrade data changes
*/
class Data implements DataPatchInterface
{
    /**
     * @var ModuleDataSetupInterface $moduleDataSetup
     */
    private $moduleDataSetup;


	/**
     * EavSetupFactory
     *
     * @var EavSetupFactory
     */
	private $eavSetupFactory;

    private $customerSetupFactory;

    /**
     * @param ModuleDataSetupInterface $moduleDataSetup
     */
	    /**
     *
     * @param EavSetupFactory          $eavSetupFactory
     */
    public function __construct(ModuleDataSetupInterface $moduleDataSetup,CustomerSetupFactory $customerSetupFactory,EavSetupFactory $eavSetupFactory,\Magento\Eav\Api\AttributeRepositoryInterface $attributeRepository)
    {
        $this->moduleDataSetup = $moduleDataSetup;
        $this->customerSetupFactory = $customerSetupFactory;
		$this->eavSetupFactory = $eavSetupFactory;
		$this->attributeRepository = $attributeRepository;
    }

    /**
     * Do Upgrade
     *
     * @return void
     */
    public function apply()
    {   

        $customerSetup = $this->customerSetupFactory->create(['setup' => $this->moduleDataSetup]);

		$this->moduleDataSetup->startSetup();

	        /**
	         * stripe_customer_id customer attribute: Stores Stripe customer ID for each customer.
	         */
	        $customerSetup->addAttribute(
	            \Magento\Customer\Model\Customer::ENTITY,
	            'quivers_stripe_customer_id',
	            [
	                'label'            => 'Quivers Stripe Customer ID',
	                'type'             => 'varchar',
	                'input'            => 'text',
	                'default'          => '',
	                'position'         => 70,
	                'visible'          => false,
	                'required'         => false,
	                'system'           => false,
	                'user_defined'     => true,
	                'visible_on_front' => false,
	            ]
	        );

	        $profileIdAttr = $customerSetup->getEavConfig()->getAttribute(
	            \Magento\Customer\Model\Customer::ENTITY,
	            'quivers_stripe_customer_id'
	        );

	        $profileIdAttr->addData([
	            'attribute_set_id' => $customerSetup->getDefaultAttributeSetId(\Magento\Customer\Model\Customer::ENTITY),
	            'attribute_group_id' => $customerSetup->getDefaultAttributeGroupId(\Magento\Customer\Model\Customer::ENTITY),
	            'used_in_forms' => [],
	        ]);

	        $this->attributeRepository->save($profileIdAttr);

	        $this->moduleDataSetup->endSetup();


		$customerSetup = $this->customerSetupFactory->create(['setup' => $this->moduleDataSetup]);

		$this->moduleDataSetup->startSetup();
			/**
			 * stripe_customer_cardIds customer attribute: Stores Stripe CardIds for each customer.
			 */
		$customerSetup->addAttribute(
			\Magento\Customer\Model\Customer::ENTITY,
			'quivers_stripe_card_ids',
			[
				'label'            => 'Quivers Stripe Cards Ids',
				'type'             => 'varchar',
				'input'            => 'text',
				'default'          => '',
				'position'         => 80,
				'visible'          => false,
				'required'         => false,
				'system'           => false,
				'user_defined'     => true,
				'visible_on_front' => false,
			]
		);

		$profileIdAttr = $customerSetup->getEavConfig()->getAttribute(
			\Magento\Customer\Model\Customer::ENTITY,
			'quivers_stripe_card_ids'
		);

		$profileIdAttr->addData([
			'attribute_set_id' => $customerSetup->getDefaultAttributeSetId(\Magento\Customer\Model\Customer::ENTITY),
			'attribute_group_id' => $customerSetup->getDefaultAttributeGroupId(\Magento\Customer\Model\Customer::ENTITY),
			'used_in_forms' => [],
		]);

		$this->attributeRepository->save($profileIdAttr);

		$this->moduleDataSetup->endSetup();	


        $eavSetup = $this->eavSetupFactory->create(['setup' => $this->moduleDataSetup]);

	        $eavSetup->addAttribute(
				\Magento\Catalog\Model\Product::ENTITY,
				'product_tax_category',
				[
					'type' => 'text',
					'backend' => '',
					'frontend' => '',
					'label' => 'Product Tax Code',
					'input' => 'text',
					'class' => '',
					'source' => '',
					'global' => \Magento\Eav\Model\Entity\Attribute\ScopedAttributeInterface::SCOPE_GLOBAL,
					'visible' => true,
					'required' => false,
					'sort_order' => 900,
					'user_defined' => false,
					'default' => '',
					'searchable' => false,
					'filterable' => false,
					'comparable' => false,
					'visible_on_front' => false,
					'used_in_product_listing' => true,
					'unique' => false,
					'apply_to' => ''
				]
			);
			$eavSetup->addAttribute(
				\Magento\Catalog\Model\Product::ENTITY,
				'quivers_claiming_group',
				[
					'type' => 'text',
					'backend' => '',
					'frontend' => '',
					'label' => 'Quivers Claiming Group',
					'input' => 'text',
					'class' => '',
					'source' => '',
					'global' => \Magento\Eav\Model\Entity\Attribute\ScopedAttributeInterface::SCOPE_GLOBAL,
					'visible' => true,
					'required' => false,
					'sort_order' => 900,
					'user_defined' => false,
					'default' => '',
					'searchable' => false,
					'filterable' => false,
					'comparable' => false,
					'visible_on_front' => false,
					'used_in_product_listing' => true,
					'unique' => false,
					'apply_to' => ''
				]
			);
		     	$eavSetup->addAttribute(
				   \Magento\Catalog\Model\Product::ENTITY,
				    'Pre_Order_Availability',
					[    
						'group' => 'General',
						'type' => 'int',
						'backend' => '',
						'frontend' => '',
						'label' => 'Pre-Order Availability',
						'input' => 'select',
						'class' => '',
						'source' => 'Quivers\Integration\Model\Config\Source\PreOrder',
						'global' => \Magento\Eav\Model\Entity\Attribute\ScopedAttributeInterface::SCOPE_GLOBAL,
						'visible' => true,
						'required' => false,
						'sort_order' => 900,
						'wysiwyg_enabled'   => false,
						'default' => '',
                        'visible_on_front' => false,
						'used_in_product_listing' => true
					]
				    );

				$eavSetup->addAttribute(
					\Magento\Catalog\Model\Product::ENTITY,
					'upc',
					[
						'type' => 'text',
						'backend' => '',
						'frontend' => '',
						'label' => 'UPC Code',
						'input' => 'text',
						'class' => '',
						'source' => '',
						'global' => \Magento\Eav\Model\Entity\Attribute\ScopedAttributeInterface::SCOPE_GLOBAL,
						'visible' => true,
						'required' => false,
						'sort_order' => 900,
						'user_defined' => false,
						'default' => '',
						'searchable' => false,
						'filterable' => false,
						'comparable' => false,
						'visible_on_front' => false,
						'used_in_product_listing' => true,
						'unique' => false,
						'apply_to' => '',
					]);

					$eavSetup->addAttribute(
						\Magento\Catalog\Model\Product::ENTITY,
						'backend_upc',
						[
							'type' => 'text',
							'backend' => '',
							'frontend' => '',
							'label' => 'UPC',
							'input' => 'text',
							'class' => '',
							'source' => '',
							'global' => \Magento\Eav\Model\Entity\Attribute\ScopedAttributeInterface::SCOPE_GLOBAL,
							'visible' => false,
							'required' => false,
							'sort_order' => 900,
							'user_defined' => false,
							'default' => '',
							'searchable' => false,
							'filterable' => false,
							'comparable' => false,
							'visible_on_front' => false,
							'used_in_product_listing' => true,
							'unique' => false,
							'apply_to' => '',
						]);    

    }

    /**
     * @inheritdoc
     */
    public function getAliases()
    {
        return [];
    }

    /**
     * @inheritdoc
     */
    public static function getDependencies()
    {
        return [
          //  UpdateIdentifierCustomerAttributesVisibility::class,
        ];
    }
	public static function getVersion()
	{
  		return '2.3.6';
	}
}